package org.rcisoft.business.bchapter.controller;

import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.rcisoft.business.bchapter.dto.QueryScoreListDTO;
import org.rcisoft.business.bchapter.dto.ScoreInfoDTO;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.enums.IsCompleteEnum;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.brslstudent.service.BRSlStudentService;
import org.rcisoft.business.bsl.service.BSlService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.enums.RoleEnum;
import org.rcisoft.core.enums.RoleFlagEnum;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UploadUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.async.WebAsyncTask;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.Callable;

/**
 * Created by gwf on 2017-7-21 15:08:47.
 */
@RestController
@RequestMapping("/BChapter")
@Slf4j
public class BChapterController extends PaginationController<BChapter> {
    @Autowired
    private BChapterService bChapterService;
    @Autowired
    private BSlService bSlService;
    /*@Autowired
    private IAction action;*/
    @Autowired
    private BRSlStudentService brSlStudentService;

    @Autowired
    private Global global;

    @ApiOperation(value="添加/编辑章节", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterName", value = "章节名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterLevel", value = "等级，1：章 2：节", required = false, dataType = "Byte"),
            @ApiImplicitParam(name = "pid", value = "上一级", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterUrl", value = "课程url", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "mdFile", value = "任务书", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isTest", value = "1:实验  2:视频   3:PPT'", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(@Valid BChapter bChapter,
                      BindingResult bindingResult
    ) {
        if (bindingResult.hasErrors()) {
            if (bChapter.getBusinessId() == null)
                throw new ServiceException(ResultServiceEnums.PARAMER_ERROR.getCode(),
                        bindingResult.getFieldError().getDefaultMessage());
        }
        PersistModel data = bChapterService.persist(bChapter,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bChapter);
    }


    @ApiOperation(value="删除章节", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bChapterService.removeBChapter(id,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="唯一查找", notes="根据主键唯一查找任务书")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/getMdFileById")
    public Result queryOne(String businessId) {
        String mdFile = bChapterService.selectOne(businessId).getMdFile();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                mdFile);
    }

    @ApiOperation(value="上传图片", notes="上传任务书所需图片")
    @ApiImplicitParam(name = "image", value = "图片文件", required = true, dataType = "MultipartFile")
    @PostMapping(value = "/uploadMdFileWithoutAuth")
    public Map<String, Object> uploadMdFile(@RequestParam(name = "editormd-image-file") MultipartFile image) {
        String path = global.getMD_FILE_LOCATION();
        String basePath = global.getBASE_UPLOAD_SERVER_LOCATION();
        Map<String, Object> pic = UploadUtil.picImport(basePath, path, image);
        Map<String, Object> result = new HashMap<>();
        if (!(boolean) pic.get(UploadUtil.IS_SUCCESSS)) {
            result.put("success", 0);
            result.put("message", ResultServiceEnums.UPLOAD_FILE_ERROR.getMessage());
            result.put("url", "");
        } else {
            result.put("url", global.getSERVER_URL() + "/" + pic.get(UploadUtil.URL));
            result.put("success", 1);
            result.put("message", ResultServiceEnums.SUCCESS.getMessage());
        }
        return result;
    }


    @ApiOperation(value="分页查询章节", notes="根据条件分页查询章节")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterName", value = "章节名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterLevel", value = "等级，1：章 2：节", required = false, dataType = "Byte"),
            @ApiImplicitParam(name = "pid", value = "上一级", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterUrl", value = "课程url", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "mdFile", value = "任务书", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isTest", value = "1:实验  2:视频   3:PPT'", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PostMapping(value = "/queryBChaptersByPagination")
    public GridModel queryBChaptersByPagination(BChapter param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bChapterService.queryBChaptersByPagination(getPaginationUtility(), param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }


    @ApiOperation(value="根据条件查询章节", notes="根据条件查询章节")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterName", value = "章节名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterLevel", value = "等级，1：章 2：节", required = false, dataType = "Byte"),
            @ApiImplicitParam(name = "pid", value = "上一级", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterUrl", value = "课程url", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "mdFile", value = "任务书", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isTest", value = "1:实验  2:视频   3:PPT'", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBChaptersWithoutAuth")
    public Result queryBChapters(BChapter param) {
        /*if (StringUtils.isEmpty(param.getSlId()))
            throw new ServiceException(ResultServiceEnums.PARAMER_ERROR);
        Map<String, Object> map = new HashedMap();
        boolean islogin = UserUtil.isLogin();
        if (!islogin)
            map.put("RoleFlag", RoleFlagEnum.TOURIST.getFlag());
        else {
            User user = UserUtil.getUser();
            if (user.getRole().getCode().equals(RoleEnum.TEACHER.getCode()))
                map.put("RoleFlag", RoleFlagEnum.TEACHER.getFlag());
            else {
                map.put("RoleFlag", RoleFlagEnum.STUDENT.getFlag());
                map.put("scoreList", bChapterService.queryScoreListBySlIdAndStuId(param.getSlId(), user.getBusinessId()));
            }
        }*/
        Map<String, Object> map = new HashedMap();
        map.put("chapterList", bChapterService.queryBChapters(param));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                map);
    }

    @ApiOperation(value="分数列表", notes="根据章节id获取分数列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "stuInfo", value = "学生学号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterId", value = "章节主键", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryScoreListByChapterId")
    public GridModel queryScoreListByChapterId(QueryScoreListDTO dto) {
        bChapterService.queryScoreListByChapterIdByPagination(getPaginationUtility(), dto);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="批量打分", notes="批量打分")
    @ApiImplicitParams({@ApiImplicitParam(name = "chapterId", value = "课程id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "score", value = "分数", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "studentId", value = "学生id", required = false, dataType = "varchar")})
    @PostMapping(value = "/markBatch")
    public Result markBatch(String scoreInfoList) {
        Gson gson = new Gson();
        List<ScoreInfoDTO> list = new ArrayList<ScoreInfoDTO>();
        try {
            list = gson.fromJson(scoreInfoList,
                    new TypeToken<List<ScoreInfoDTO>>() {
                    }.getType());
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException(ResultServiceEnums.PARAMER_ERROR);
        }
        bChapterService.markScore(list,getToken());
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                scoreInfoList);
    }

    @ApiOperation(value="重新打分", notes="重新打分")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/reMark")
    public Result reMark(String id) {
        bChapterService.deleteScoerInfo(id);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="完成学习", notes="完成学习")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "score", value = "分数", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterId", value = "章节id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "studentId", value = "学生学号", required = false, dataType = "varchar")})
    @PostMapping(value = "/complete")
    public Result complete(ScoreInfoDTO scoreInfoDTO) {
        scoreInfoDTO.setIsComplete(IsCompleteEnum.COMPLETE.getCode());
        /*User user = UserUtils.getUser();
        if (!user.getRole().getCode().equals(RoleEnum.STUDENT.getCode()))
            throw new ServiceException(ResultServiceEnums.ROLE_ERROR);*/

        scoreInfoDTO.setStudentId(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bChapterService.updateIsComplete(scoreInfoDTO,getToken());
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                scoreInfoDTO);
    }

    @ApiOperation(value="启动che", notes="启动che")
    @ApiImplicitParams({@ApiImplicitParam(name = "teaCode", value = "教师编号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "stuCode", value = "学生编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "flag", value = "用户身份", required = false, dataType = "varchar")})
    @PostMapping(value = "/cheStart")
    public WebAsyncTask<Result> cheStart(String teaCode,
                                               String stuCode,
                                               String slCode,
                                               Integer flag) throws Exception {
        Result result = new Result();
        log.info("主线程开始");
        Callable<Result> data = (() -> {
            log.info("副线程开始");
            String port = getPort();
            String url = null, password = null, project = null, username = null, gitUrl = null, email = null;
            //1.判断用户身份
            if (flag == null || (flag != 0 && flag != 1))
                throw new ServiceException(ResultServiceEnums.PARAMER_ERROR);
            if (flag == 1) {
                if(!bSlService.stuHasSl(stuCode,slCode))
                    throw new ServiceException(ResultServiceEnums.STUDENT_HAS_NOT_SL);
                password = stuCode;
                username = global.getGIT_LAB_STU_PREFIX() + stuCode;
                gitUrl = "/" + global.getGIT_LAB_TEA_PREFIX() + teaCode + "/" + global.getGIT_LAB_PROJECT_PREFIX() + slCode + ".git";
            } else if (flag == 0) {
                if(!bSlService.teaHasSl(teaCode, slCode))
                    throw new ServiceException(ResultServiceEnums.TEACHER_HAS_NOT_SL);
                password = teaCode;
                username = global.getGIT_LAB_TEA_PREFIX() + teaCode;
                gitUrl = "/" + global.getGIT_LAB_TEA_PREFIX() + teaCode + "/" + global.getGIT_LAB_PROJECT_PREFIX() + slCode + ".git";
            }
            project = global.getGIT_LAB_PROJECT_PREFIX() + slCode;

            //2.判断当前che的数量
            /*if (CountDown.portMap.size() >= Integer.parseInt(global.getCHE_MAX_NUMBER()))
                throw new ServiceException(ResultServiceEnums.CHE_NUM_MAX);
            //3.判断当前用户有没有已经开启的che
            if (CountDown.useChe.containsKey(username)) {
                if (!CountDown.useChe.get(username).equals(project)) {
                    throw new ServiceException(ResultServiceEnums.CHE_HAS_START);
                } else {
                    Map<String, String> map = CountDown.projectMap.get(project);
                    result.setCode(ResultCode.SUCCESS.code);
                    result.setMessage(ResultServiceEnums.SUCCESS.getMessage());
                    result.setData(map);
                    return result;
                }
            }
            email = UserUtil.getUser().getEmail();
            url = action.cheStart(project, password, username, gitUrl, port, email);
            Map<String, String> map = new HashMap<>();
            map.put("url", url);
            map.put("port", port);
            map.put("endTime", String.valueOf(CountDown.portMap.get(port).getEndTime().getTime()));
            result.setCode(ResultCode.SUCCESS.code);
            result.setMessage(ResultServiceEnums.SUCCESS.getMessage());
            result.setData(map);
            log.info("副线程结束");*/
            return result;
        });
        WebAsyncTask<Result> webAsyncTask = new WebAsyncTask<>(300000, data);
        webAsyncTask.onTimeout(timeOutCallBack());
        log.info("主线程结束");
        return webAsyncTask;
    }

    private Callable<Result> timeOutCallBack(){

        Callable<Result> callback = (()-> {
                log.info("请求超时");
                Result result = new Result();
                result.setMessage("请求超时");
                return result;
        });
        return callback;
    }

    @ApiOperation(value="关闭che", notes="根据端口关闭che")
    @ApiImplicitParam(name = "port", value = "che启动的端口", required = true, dataType = "varchar")
    @PostMapping(value = "/cheStopWithoutAuth" )
    public WebAsyncTask<Result> cheStop(String port) throws Exception {
        Result result = new Result();
        log.info("主线程开始");
        Callable<Result> data = (() -> {
            log.info("副线程开始");
            /*action.cheStop(port);
            result.setCode(ResultCode.SUCCESS.code);
            result.setMessage(ResultServiceEnums.SUCCESS.getMessage());
            result.setData(null);*/
            log.info("副线程结束");
            return result;
        });
        WebAsyncTask<Result> webAsyncTask = new WebAsyncTask<>(300000, data);
        webAsyncTask.onTimeout(timeOutCallBack());
        log.info("主线程结束");
        return webAsyncTask;
    }

    @ApiOperation(value="提交代码", notes="提交代码")
    @ApiImplicitParams({@ApiImplicitParam(name = "port", value = "端口", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "stuCode", value = "学生学号", required = false, dataType = "varchar")})
    @PostMapping(value = "/pushCode")
    public Result pushCode(String port,
                                 String slCode,
                                 String stuCode) {
        /*try {
            action.pushCode(
                    global.getGIT_LAB_PROJECT_PREFIX() + slCode,
                    port,
                    global.getGIT_LAB_STU_PREFIX() + stuCode
            );*/
            return Result.builder(new PersistModel(1),
                    ResultServiceEnums.SUCCESS.getMessage(),
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    null);
        /*} catch (Exception e) {
            throw new ServiceException(ResultServiceEnums.PUSH_ERROR);
        }*/

    }

    @ApiOperation(value="检出分支", notes="检出分支")
    @ApiImplicitParams({@ApiImplicitParam(name = "port", value = "端口", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "stuCode", value = "学生学号", required = false, dataType = "varchar")})
    @PostMapping(value = "/checkoutCode")
    public Result checkoutCode(String port,
                                     String slCode,
                                     String stuCode) throws IOException, InterruptedException {
        /*action.checkoutBranch(
                global.getGIT_LAB_PROJECT_PREFIX() + slCode,
                port,
                global.getGIT_LAB_STU_PREFIX() + stuCode);*/
        return Result.builder(new PersistModel(1),
                ResultServiceEnums.SUCCESS.getMessage(),
                MessageConstant.MESSAGE_ALERT_ERROR,
                null);
    }


    @PostMapping(value = "/testCountDown")
    public String testCountDown() throws Exception {
        return getPort();
    }

    @ApiOperation(value="刷新che启动时间", notes="根据端口刷新che启动时间")
    @ApiImplicitParam(name = "port", value = "che启动的端口", required = true, dataType = "varchar")
    @PostMapping(value = "/refreshChe")
    public Result refreshChe(String port) {
        /*CountDown countDown = CountDown.portMap.get(port);
        if (countDown.getMidTime() > 300)
            throw new ServiceException(ResultServiceEnums.TEST_NOT_OVER);
        countDown.setMidTime(Integer.parseInt(global.getCHE_START_TIME()));
        return Result.builder(new PersistModel(1),ResultServiceEnums.SUCCESS.getMessage(),countDown.getEndTime().getTime());*/
        return Result.builder(new PersistModel(1),
                ResultServiceEnums.SUCCESS.getMessage(),
                MessageConstant.MESSAGE_ALERT_ERROR,
                new Date().getTime());
    }


    private String getPort() throws Exception {
        Random random = new Random();
        Integer port = 8000 + random.nextInt(1000);
        /*while (CountDown.portMap.containsKey(port))
            port = 8000 + random.nextInt(1000);*/
        return port.toString();
    }
}
