package org.rcisoft.business.bchapter.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bchapter.dto.QueryScoreListDTO;
import org.rcisoft.business.bchapter.dto.ScoreInfoDTO;
import org.rcisoft.business.bchapter.dto.StuScoreDTO;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.vo.StudentChapterScoreVO;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;
import tk.mybatis.mapper.common.Mapper;

import java.util.List;
import java.util.Map;

/**
* Created with gwf on 2017-7-21 15:08:47.
*/
@Repository
public interface BChapterRepository extends BaseMapper<BChapter> {

    /**
    * 根据条件查找
    * @param param
    * @return
    */
    @Select("<script>select * from b_chapter  where del_flag = 0 and flag = 1 and chapter_level = 1 and sl_id = #{slId} order by create_date</script>")
    @ResultMap(value = "SupperChildListResultMap" )
    List<BChapter> queryBChapters(Map<String, Object> param);

    /**
     * 根据父节点查找子节点列表
     * @param pid
     * @return
     */
    @Select("<script>SELECT\n" +
            " bc.*,\n" +
            " (SELECT bv.video_url FROM b_video bv WHERE bv.business_id = bcv1.video_id) AS videoUrl,\n" +
            " (SELECT bv.video_url FROM b_video bv WHERE bv.business_id = bcv2.video_id) AS fileUrl,\n" +
            " (SELECT count(*) FROM b_r_student_chapter brs WHERE brs.chapter_id = bc.business_id AND brs.score = - 1) AS studentNum \n" +
            " FROM\n" +
            " b_chapter bc\n" +
            " LEFT JOIN b_r_chapter_video bcv1 ON bcv1.chapter_id = bc.business_id and bcv1.type = '0'\n" +
            " LEFT JOIN b_r_chapter_video bcv2 ON bcv2.chapter_id = bc.business_id and bcv2.type = '1'\n" +
            " \n" +
            " WHERE\n" +
            " bc.del_flag = 0\n" +
            " AND bc.pid = #{pid}\n" +
            " ORDER BY\n" +
            " bc.create_date</script>")
    @ResultMap(value = "SupperChildListResultMap" )
    List<BChapter> queryBChaptersByPid(String pid);

    /**
     * 根据章节id查找该章节学生分数列表
     * @param
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tt1.chapter_name,t5.class_name,t3.`code`,t6.`name`,t2.score,t3.business_id as sId,t2.business_id as scoreId\n" +
            "FROM\n" +
            "\tb_chapter t1\n" +
            "\tLEFT JOIN b_r_student_chapter t2 ON t2.chapter_id = t1.`business_id`\n" +
            "\tLEFT JOIN b_student t3 ON t2.student_id = t3.`business_id`\n" +
            "\tLEFT JOIN b_r_class_student t4 ON t3.`code` = t4.student_code\n" +
            "\tLEFT JOIN b_class t5 ON t5.`code` = t4.class_code\n" +
            "\tLEFT JOIN s_user t6 ON t6.login_name = t3.`code`\n" +
            "WHERE t1.del_flag='0' and t1.flag='1' and t2.is_complete='1' " +
            "\nand t2.`score`  <if test=\"hasMark\">!=</if> <if test=\"!hasMark\">=</if> -1" +
            "\tand t1.business_id = #{dto.chapterId}\n" +
            "\tand (" +
            "(t3.code like #{dto.stuInfo} or t6.name like #{dto.stuInfo})\n" +
            "\tor " +
            "(t5.code like #{dto.stuInfo} or t5.class_name like #{dto.stuInfo})" +
            ")</script>")
    @ResultMap("QueryScoreListByChapterIdResultMap")
    List<StudentChapterScoreVO> queryScoreListByChapterId(@Param("dto") QueryScoreListDTO dto, @Param("hasMark") boolean hasMark);

    /**
     * 根据课程id和学生id查找该学生所学课程分数列表
     * @param slid
     * @param stuid
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tt2.chapter_id as chapterId,\n" +
            "\tt2.score\n" +
            "FROM\n" +
            "\tb_chapter t1\n" +
            "LEFT JOIN b_r_student_chapter t2 ON t1.business_id = t2.chapter_id\n" +
            "LEFT JOIN b_sl t3 ON t1.sl_id = t3.business_id\n" +
            "LEFT JOIN b_student t4 ON t2.student_id = t4.business_id\n" +
            "LEFT JOIN s_user t5 ON t5.login_name = t4.code\n" +
            "WHERE\n" +
            "\tt3.business_id = #{slid}\n" +
            "AND t5.business_id = #{stuid}</script>")
    List<StuScoreDTO> selectScoreListBySlIdAndStuId(@Param("slid") String slid, @Param("stuid") String stuid);

    @Delete("<script>delete from b_r_student_chapter where chapter_id = #{chapterId}</script>")
    int deleteStudentChapter(String chapterId);

    /**
     * 根据章节id和学生id获取主键
     * @param chapterId
     * @param studentId
     * @return
     */
    @Select("<script>select business_id from b_r_student_chapter where chapter_id = #{chapterId} and student_id = #{studentId}</script>")
    String selectIdByStudentIdAndChapterId(@Param("chapterId") String chapterId, @Param("studentId") String studentId);

    /**
     * 插入分数
     * @param scoreInfoDTO
     * @return
     */
    @Insert("</script>insert into b_r_student_chapter(business_id,chapter_id,student_id,score) values(#{businessId},#{chapterId},#{studentId},#{score})</script>")
    int insertScoreInfo(ScoreInfoDTO scoreInfoDTO);

    /**
     * 更新分数
     * @param scoreInfoDTO
     * @return
     */
    @Update("<script>update b_r_student_chapter set chapter_id=#{chapterId},student_id=#{studentId},score=#{score} where business_id = #{businessId}</script>")
    int updateScoreInfo(ScoreInfoDTO scoreInfoDTO);

    /**
     * 更新完成状态
     * @param scoreInfoDTO
     * @return
     */
    @Insert("<script>insert into b_r_student_chapter(business_id,student_id,chapter_id,is_complete) values (#{businessId},#{studentId},#{chapterId},#{isComplete})</script>")
    int updateIsComplete(ScoreInfoDTO scoreInfoDTO);

    /**
     * 删除分数记录
     * @param id
     * @return
     */
    @Update("<script>update b_r_student_chapter set score = -1 where  business_id = #{businessId}</script>")
    int deleteScoerInfo(String id);

}

