package org.rcisoft.business.bchapter.service.impl;

import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.business.bchapter.dto.QueryScoreListDTO;
import org.rcisoft.business.bchapter.dto.ScoreInfoDTO;
import org.rcisoft.business.bchapter.dto.StuScoreDTO;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.enums.ChapterLevelEnum;
import org.rcisoft.business.bchapter.enums.HasMarkEnum;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bchapter.vo.StudentChapterScoreVO;
import org.rcisoft.business.brslstudent.dao.BRSlStudentRepository;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.bstudent.dao.BStudentRepository;
import org.rcisoft.common.component.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by gwf on 2017-7-21 15:08:47.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BChapterServiceImpl implements BChapterService {
    @Autowired
    private BChapterRepository bChapterRepository;
    @Autowired
    private BSlRepository bSlRepository;
    @Autowired
    private BStudentRepository bStudentRepository;
    @Autowired
    private BRSlStudentRepository brSlStudentRepository;

    @Override
    public BChapter selectOne(String businessId){
        BChapter result = bChapterRepository.selectByPrimaryKey(businessId);
        if(result==null)
            throw new ServiceException(ResultServiceEnums.CHAPTER_NOT_EXISTS);
        return result;
    }

    @Override
    public List<BChapter> queryBChaptersByPagination(PageUtil pageUtil, BChapter model) {
        Map param = new HashMap<String,Object>();
        return bChapterRepository.queryBChapters(param);
    }

    @Override
    public List<StudentChapterScoreVO> queryScoreListByChapterIdByPagination(PageUtil pageUtil, QueryScoreListDTO dto){
        boolean result = dto.getHasMark() == HasMarkEnum.HAS_MARK.getCode();
        dto.setClassInfo(dto.getClassInfo()==null?"%%":"%"+dto.getClassInfo()+"%");
        dto.setStuInfo(dto.getStuInfo()==null?"%%":"%"+dto.getStuInfo()+"%");
        return bChapterRepository.queryScoreListByChapterId(dto,result);
    }

    @Override
    public List<BChapter> queryBChapters(BChapter model) {
        Map param = new HashMap<String,Object>();
        param.put("slId",model.getSlId());
        return bChapterRepository.queryBChapters(param);
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel persist(BChapter model, String token){
        int line = 0;
        if(model.getBusinessId()==null) {
            if (bSlRepository.selectByPrimaryKey(model.getSlId()) == null)
                throw new ServiceException(ResultServiceEnums.SL_INFO_NOT_EXISTS);
            UserUtil.setCurrentPersistOperation(model,token);
            line = bChapterRepository.insertSelective(model);
        }else
            UserUtil.setCurrentMergeOperation(model,token);
            line = bChapterRepository.updateByPrimaryKeySelective(model);
        //FamilyLogger.sysInfo(UserUtils.getUser().getBusinessId(),UserUtils.getUser().getLoginName()+"新增了ID为"+user.getBusinessId()+"的用户");
        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel update(BChapter model) {
        int line = bChapterRepository.updateByPrimaryKeySelective(model);
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel removeBChapter(String id,String token) {
        BChapter bChapter = bChapterRepository.selectByPrimaryKey(id);
        if(bChapter==null)
            throw new ServiceException(ResultServiceEnums.CHAPTER_NOT_EXISTS);
        List<BChapter> childList = bChapterRepository.queryBChaptersByPid(id);
        boolean hasChild =childList!=null&&childList.size()>0;
        boolean isChapter = bChapter.getChapterLevel()== ChapterLevelEnum.CHAPTER.getCode();
        if(isChapter&&hasChild)
            throw new ServiceException(ResultServiceEnums.CHAPTER_HAS_CHILD);
        bChapterRepository.deleteStudentChapter(id);
        bChapterRepository.deleteByPrimaryKey(id);
        UserUtil.setCurrentMergeOperation(bChapter,token);
        int line = bChapterRepository.updateByPrimaryKeySelective(bChapter);
        return new PersistModel(line);
    }

    @Override
    @Transactional
    public void markScore(List<ScoreInfoDTO> scoreInfoDTOS,String token) {
        for(ScoreInfoDTO scoreInfoDTO:scoreInfoDTOS){
            String id = bChapterRepository.selectIdByStudentIdAndChapterId(scoreInfoDTO.getChapterId(),scoreInfoDTO.getStudentId());
            if(scoreInfoDTO.getScore()==null||scoreInfoDTO.getScore()<0||scoreInfoDTO.getScore()>100)  //分数不为空且在0到100之间
                throw new ServiceException(ResultServiceEnums.SCORE_ERROR);
            if(id==null){   //查看该学生该章节是否已经完成学习
                  throw new ServiceException(ResultServiceEnums.UN_COMPLETE);
//                BStudent b;
//                if((b=bStudentRepository.selectByPrimaryKey(scoreInfoDTO.getStudentId()))==null) {  //查看学生是否存在
//                    throw new BusinessException(ResultEnums.STUDENT_NOT_EXISTS);
//                }
//                BChapter bChapter;
//                if((bChapter = bChapterRepository.selectByPrimaryKey(scoreInfoDTO.getChapterId()))==null)  //查看章节是否存在
//                    throw new BusinessException(ResultEnums.CHAPTER_NOT_EXISTS);
//                if(brSlStudentRepository.selectByStudentIdAndSlId(b.getCode(),bChapter.getSlId())==null)  //查看该学生是否已经选择该门课程
//                    throw new BusinessException(ResultEnums.B_R_SL_STUDENT_NOT_EXISTS);
//                scoreInfoDTO.setCommonBusinessId();   //未打分的话应该新添加该章节分数记录
//                bChapterRepository.insertScoreInfo(scoreInfoDTO);
            }else{   //已经学习则修改
                scoreInfoDTO.setBusinessId(id);
                UserUtil.setCurrentMergeOperation(scoreInfoDTO,token);
                bChapterRepository.updateScoreInfo(scoreInfoDTO);
            }
        }
    }

    @Override
    public List<StuScoreDTO> queryScoreListBySlIdAndStuId(String slid, String stuid) {
        return bChapterRepository.selectScoreListBySlIdAndStuId(slid, stuid);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public void deleteScoerInfo(String id) {
        int result = bChapterRepository.deleteScoerInfo(id);
        if(result<1)
            throw new ServiceException(ResultServiceEnums.SCORE_INFO_NOT_EXISTS);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public void updateIsComplete(ScoreInfoDTO scoreInfoDTO,String token) {
        String stuid = bStudentRepository.queryStudentByUserId(scoreInfoDTO.getStudentId());
        scoreInfoDTO.setStudentId(stuid);
        if(bChapterRepository.selectIdByStudentIdAndChapterId(scoreInfoDTO.getChapterId(),scoreInfoDTO.getStudentId())!=null)
            return;
        scoreInfoDTO.setCommonBusinessId();
        UserUtil.setCurrentMergeOperation(scoreInfoDTO,token);
        int result = bChapterRepository.updateIsComplete(scoreInfoDTO);
        if(result<1)
            throw new ServiceException(ResultServiceEnums.UPDATE_ERROR);
    }
}
