package org.rcisoft.business.blesson.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.Get;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.service.BLessonService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.converter.MultipartFile2HSSFWorkbookConverter;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UploadUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Map;

import static org.springframework.web.bind.annotation.RequestMethod.GET;

/**
* Created by CodeGenerator on 2017/07/12.
*/
@RestController
@RequestMapping("/BLesson")
public class BLessonController extends PaginationController<BLesson> {
    @Autowired
    private BLessonService bLessonService;

    @Autowired
    private Global global;


    @ApiOperation(value="添加/编辑课程", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonName", value = "课程名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "defaultUrl", value = "默认封面图片url", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(@Valid BLesson bLesson, BindingResult bindingResult) {
        if(bLesson.getCode().length()>
                Integer.parseInt(global.getMAX_CODE_LENGTH()))
            throw new ServiceException(ResultServiceEnums.PARAMER_ERROR);
        if (bindingResult.hasErrors()) {
            if (bLesson.getBusinessId() == null)
                throw new ServiceException(ResultServiceEnums.PARAMER_ERROR.getCode(),
                        bindingResult.getFieldError().getDefaultMessage());
        }
        PersistModel data = bLessonService.persist(bLesson,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLesson);
    }

    @ApiOperation(value="删除课程", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bLessonService.removeBLesson(id);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="excel导入", notes="上传excel到服务器")
    @ApiImplicitParam(name = "importFile", value = "excel文件", required = false, dataType = "MultipartFile")
    @PostMapping(value = "excelImport")
    public Result excelImport(MultipartFile importFile) throws Exception {
        PersistModel data = bLessonService.importExcel(MultipartFile2HSSFWorkbookConverter.convert(importFile),getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data.getInfluenceReason());
    }

    @ApiOperation(value="上传图片", notes="上传图片到服务器")
    @ApiImplicitParams({@ApiImplicitParam(name = "importFile", value = "图片文件", required = false, dataType = "MultipartFile"),
            @ApiImplicitParam(name = "code", value = "课程编号", required = false, dataType = "varchar")})
    @PostMapping(value = "picImport")
    public Result picImport(MultipartFile importFile,String code) throws Exception {
        String path = global.getBASE_UPLOAD_SERVER_LOCATION();
        String course_logo_path =global.getIMAGE_LOCATION()+global.getCOURCE_LOGO_LOCATION();
        Map<String,Object> map= UploadUtil.picImport(path,course_logo_path,importFile);
        if(map.get(UploadUtil.IS_SUCCESSS).equals(true))
        {
            BLesson bL=new BLesson();
            bL.setCode(code);
            BLesson bLesson=bLessonService.selectOne(bL);
            if(bLesson!=null){
                bLesson.setDefaultUrl(map.get(UploadUtil.URL).toString());
                UserUtil.setCurrentMergeOperation(bLesson,getToken());
                int line = bLessonService.updateByPrimaryKeySelective(bLesson);
                return Result.builder(new PersistModel(line),
                        MessageConstant.MESSAGE_ALERT_SUCCESS,
                        MessageConstant.MESSAGE_ALERT_ERROR,
                        bLessonService.selectOne(code));
            }else{
                throw new ServiceException(ResultServiceEnums.UPLOAD_FILE_ERROR);
            }
        }
        else
        {
            throw new ServiceException(ResultServiceEnums.ADD_ERROR);
        }
    }

    @ApiOperation(value="恢复默认", notes="恢复默认封面图片")
    @ApiImplicitParam(name = "code", value = "课程编号", required = false, dataType = "varchar")
    @PostMapping(value = "defaultUrl")
    public Result defaultUrl(String code)throws Exception {
            String path = global.getDEFAULT_COURSE_LOCATION();
            BLesson bL=new BLesson();
            bL.setCode(code);
            BLesson bLesson=bLessonService.selectOne(bL);
            if(bLesson!=null){
                bLesson.setDefaultUrl(path);
                UserUtil.setCurrentMergeOperation(bLesson,getToken());
                int line =bLessonService.updateByPrimaryKeySelective(bLesson);
                return Result.builder(new PersistModel(line),
                        MessageConstant.MESSAGE_ALERT_SUCCESS,
                        MessageConstant.MESSAGE_ALERT_ERROR,
                        path);
            }else{
                throw new ServiceException(ResultServiceEnums.UPLOAD_FILE_ERROR);
            }
    }



    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询", notes="根据URL中的参数分页查询")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonName", value = "课程名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "defaultUrl", value = "默认封面图片url", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "delFlag", value = "删除标记（0：正常；1：删除；2：审核）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "flag", value = "启用标记（0：停用；1：启用）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBLessonsByPagination")
    public GridModel queryBLessonsByPagination(BLesson param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bLessonService.queryBLessonsByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件查询", notes="根据URL中的参数查询全部")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonName", value = "课程名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "defaultUrl", value = "默认封面图片url", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "delFlag", value = "删除标记（0：正常；1：删除；2：审核）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "flag", value = "启用标记（0：停用；1：启用）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBLessons")
    public Result queryBLessons(BLesson param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.queryBLessons(param));
    }

}
