package org.rcisoft.business.blesson.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.service.BLessonService;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.component.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BLessonServiceImpl implements BLessonService {
    @Autowired
    private BLessonRepository bLessonRepository;
    @Autowired
    private BSlRepository bSlRepository;
    @Autowired
    private Global global;


    @Override
    public BLesson selectOne(String businessId){
        return bLessonRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public BLesson selectOne(BLesson bLesson) {
        return bLessonRepository.selectOne(bLesson);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public int  updateByPrimaryKeySelective(BLesson bLesson) {
        return bLessonRepository.updateByPrimaryKeySelective(bLesson);
    }

    @Override
    public List<BLesson> queryBLessonsByPagination(PageUtil pageUtil, BLesson model) {
        return bLessonRepository.queryBLessons(queryParamHandler(model));
    }

    @Override
    public List<BLesson> queryBLessons(BLesson model) {
        return bLessonRepository.queryBLessons(queryParamHandler(model));
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel persist(BLesson model,String token){
        int line = 0;
        if(model.getBusinessId()==null) {
            if (bLessonRepository.queryBLessonByCode(model.getCode()) != null)
                throw new ServiceException(ResultServiceEnums.INSERT_DATA_EXIST);
            model.setCommonBusinessId();
            model.setDefaultUrl(global.getDEFAULT_COURSE_LOCATION());
            UserUtil.setCurrentPersistOperation(model,token);
            line = bLessonRepository.insertSelective(model);
        }else
            UserUtil.setCurrentMergeOperation(model,token);
            line = bLessonRepository.updateByPrimaryKeySelective(model);
        //FamilyLogger.sysInfo(UserUtils.getUser().getBusinessId(),UserUtils.getUser().getLoginName()+"新增了ID为"+user.getBusinessId()+"的用户");
        return new PersistModel(line);
    }


    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel importExcel(HSSFWorkbook hwb, String token) throws IOException {
        ArrayList<BLesson> lessons = new ArrayList<BLesson>();
        ArrayList<String> repeatCode = new ArrayList<String>();
        List<String> errorCode = new ArrayList<>();

        String[] headers = {"课程编号","课程名称"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb,headers);  //获取excel数据


        if (values.size()<0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        for(String[] value:values){   //将数据封装到entity
            BLesson bLesson = null;
            String classCode = value[0];

            //判断1:班级编号是否满足<15位；
            if(classCode.length()>15){
                //该用户不满足要求，记入valuesShort中
                errorCode.add(value[0]);
                continue;
            }
            if((bLesson=bLessonRepository.queryBLessonByCode(value[0]))!=null) {
                repeatCode.add(value[0]);
                continue;
            }
            bLesson= new BLesson(value[0],value[1]);
            bLesson.setDefaultUrl(global.getDEFAULT_COURSE_LOCATION());
            UserUtil.setCurrentPersistOperation(bLesson,token);

            if(!lessons.contains(bLesson)){
                lessons.add(bLesson);
            }
        }

//        if(lessons.size()<1){
//            if(repeatCode.size()<1&&errorCode.size()<1)
//                throw new BusinessException(ResultEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
//            else
//                throw new BusinessException(ResultEnums.EXCEL_IMPORT_DATA_NOT_EXIST.getCode(),"Excel内所有数据之前都已经导入。");
//        }
        String result ="";
        if(lessons.size()>0){
            int line = bLessonRepository.insertList(lessons);

            if (line>0) {
                result+="成功导入"+lessons.size()+"门课程。";
            }else{
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR);

            }
        }


        if(repeatCode.size()>0){
            result+="以下课程编号存在重复："+ JSON.toJSONString(repeatCode)+"。";
        }
        if(errorCode.size()>0){
            result+="以下课程编号超过15位："+ JSON.toJSONString(errorCode)+"。";
        }
        if (lessons.size()<1){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST.getCode(),result);
        }

        return new PersistModel(1,result);


    }
    public PersistModel importPic(HSSFWorkbook hwb) throws IOException
    {
        return null;
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel removeBLesson(String id) {
        if(bSlRepository.selectslNumByLesson(id)>0)
            throw new ServiceException(ResultServiceEnums.LESSON_HAS_SL);
        int line = bLessonRepository.deleteByPrimaryKey(id);
        return new PersistModel(line);
    }

    private Map<String,Object> queryParamHandler(BLesson model){
        Map param = new HashMap<String, Object>();
        if(model.getLessonName()!=null)
            param.put("name","%"+model.getLessonName()+"%");
        else
            param.put("name","%%");
        if(model.getCode()!=null)
            param.put("code","%"+model.getCode()+"%");
        else
            param.put("code","%%");
        return param;
    }


}
