package org.rcisoft.business.brslstudent.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.brslstudent.entity.BRSlStudent;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
* Created with CodeGenerator on 2017/07/12.
*/
@Repository
public interface BRSlStudentRepository extends BaseMapper<BRSlStudent> {

    /**
    * 根据条件查找
    * @param param
    * @return
    */
    @Select("<script>select * from b_r_sl_student</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BRSlStudent> queryBRSlStudents(Map<String, Object> param);

    /**
     * 根据学号和课序号查询开课信息
     * @param studentCode
     * @param slCode
     * @return
     */
    @Select("<script>SELECT * FROM b_r_sl_student  WHERE  student_code = #{studentCode} AND sl_code = #{slCode}</script>")
    @ResultMap(value = "BaseResultMap" )
    BRSlStudent selectByStudentIdAndSlId(@Param("studentCode") String studentCode, @Param("slCode") String slCode);

    /**
     * 根据code查找
     * @param brSlStudent
     * @return
     */
    @Select("<script>select * from b_r_sl_student where student_code = #{studentCode} and sl_code = #{slCode}</script>")
    @ResultMap(value = "BaseResultMap" )
    BRSlStudent queryBRSlStudentByCodes(BRSlStudent brSlStudent);

    /**
     * 批量插入
     * @param brSlStudents
     * @return
     */
    @Insert("<script>INSERT INTO b_r_sl_student" +
            "(business_id,student_code,sl_code)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "(#{item.businessId},#{item.studentCode},#{item.slCode})" +
            "</foreach></script>")
    int insertList(List<BRSlStudent> brSlStudents);

    /**
     * 根据slcode删除开课信息
     * @param slCode
     * @return
     */
    @Delete("<script>delete from b_r_sl_student where sl_code = #{slCode}</script>")
    int deleteBatch(@Param("slCode") String slCode);

    /**
     * 查询学生学习课程数
     * @param stuId
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tCOUNT(b1.sl_code)\n" +
            "FROM\n" +
            "\tb_r_sl_student b1\n" +
            "LEFT JOIN b_student b2 ON b1.student_code = b2.`code`\n" +
            "WHERE\n" +
            "\tb2.business_id = #{stuId}</script>")
    int selectSlNumByStuId(String stuId);
}

