package org.rcisoft.business.bsl.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bsl.dto.SlDetailDTO;
import org.rcisoft.business.bsl.entity.BSl;
import org.rcisoft.business.bsl.service.BSlService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.converter.MultipartFile2HSSFWorkbookConverter;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UploadUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.io.UnsupportedEncodingException;
import java.util.Map;

/**
 * Created by CodeGenerator on 2017/07/12.
 */
@RestController
@RequestMapping("/BSl")
public class BSlController extends PaginationController<BSl> {
    @Autowired
    private BSlService bSlService;
    @Autowired
    private Global global;

    @ApiOperation(value="添加/编辑课程", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "课序号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "teacherCode", value = "教师编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "termCode", value = "学期编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isOnline", value = "是否上线，（0：待上线，1：已上线，2：已下线）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slCoverUrl", value = "课程封面", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slNotes", value = "课程简介", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isRecommend", value = "是否推荐", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendDate", value = "推荐日期", required = false, dataType = "datetime"),
            @ApiImplicitParam(name = "recommendOrder", value = "推荐序号", required = false, dataType = "integer"),
            @ApiImplicitParam(name = "recommendCourse", value = "推荐原因", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "gitLabProjectId", value = "gitlab项目id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "gitLabProjectName", value = "gitlab项目名", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "delFlag", value = "删除标记（0：正常；1：删除；2：审核）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "flag", value = "启用标记（0：停用；1：启用）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(@Valid BSl bSl, BindingResult bindingResult) throws UnsupportedEncodingException {
        if(bSl.getCode()!=null&&bSl.getCode().length()>
                Integer.parseInt(global.getMAX_CODE_LENGTH()))
            throw new ServiceException(ResultServiceEnums.PARAMER_ERROR);
        if (bindingResult.hasErrors()) {
            if (bSl.getBusinessId() == null)
                throw new ServiceException(ResultServiceEnums.PARAMER_ERROR.getCode(),
                        bindingResult.getFieldError().getDefaultMessage());
        }
        PersistModel data = bSlService.persist(bSl,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bSl);
    }

    @ApiOperation(value="excel导入", notes="excel导入")
    @ApiImplicitParam(name = "importFile", value = "excel文件", required = true, dataType = "MultipartFile")
    @PostMapping(value = "excelImport")
    public Result excelImport(MultipartFile importFile) throws Exception {
        //调用service层方法，进行具体的excel导入相关操作，这里先讲springMVC接收到的文件转化成HSSFWorkbook需要的excel文件
        PersistModel data = bSlService.importExcel(MultipartFile2HSSFWorkbookConverter.convert(importFile),getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data.getInfluenceReason());
    }

    @ApiOperation(value="修改课程是否上线", notes="0：待上线，1：已上线，2：已下线")
    @ApiImplicitParam(name = "code", value = "课程编号", required = false, dataType = "varchar")
    @PostMapping(value = "updateIsOnline")
    public Result updateIsOnline(String code) {
        PersistModel data = bSlService.updateIsOnline(code,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                code);
    }

    @ApiOperation(value="唯一查找", notes="根据主键唯一查找任务书")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bSlService.selectOne(businessId));
    }


    @ApiOperation(value="分页查询课程", notes="根据条件分页查询课程")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "课序号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "teacherCode", value = "教师编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "termCode", value = "学期编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isOnline", value = "是否上线，（0：待上线，1：已上线，2：已下线）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slCoverUrl", value = "课程封面", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slNotes", value = "课程简介", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isRecommend", value = "是否推荐", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendDate", value = "推荐日期", required = false, dataType = "datetime"),
            @ApiImplicitParam(name = "recommendOrder", value = "推荐序号", required = false, dataType = "integer"),
            @ApiImplicitParam(name = "recommendCourse", value = "推荐原因", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "gitLabProjectId", value = "gitlab项目id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "gitLabProjectName", value = "gitlab项目名", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "teacherName", value = "教师姓名", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonName", value = "课程名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "termName", value = "学期名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "delFlag", value = "删除标记（0：正常；1：删除；2：审核）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "flag", value = "启用标记（0：停用；1：启用）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBSlsByPagination")
    public GridModel queryBSlsByPagination(BSl param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bSlService.queryBSlsByPagination(getPaginationUtility(), param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="条件查询查询课程", notes="根据条件查询课程")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "code", value = "课序号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "teacherCode", value = "教师编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "termCode", value = "学期编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isOnline", value = "是否上线，（0：待上线，1：已上线，2：已下线）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slCoverUrl", value = "课程封面", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slNotes", value = "课程简介", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "isRecommend", value = "是否推荐", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "recommendDate", value = "推荐日期", required = false, dataType = "datetime"),
            @ApiImplicitParam(name = "recommendOrder", value = "推荐序号", required = false, dataType = "integer"),
            @ApiImplicitParam(name = "recommendCourse", value = "推荐原因", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "gitLabProjectId", value = "gitlab项目id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "gitLabProjectName", value = "gitlab项目名", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "teacherName", value = "教师姓名", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonName", value = "课程名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "termName", value = "学期名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "delFlag", value = "删除标记（0：正常；1：删除；2：审核）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "flag", value = "启用标记（0：停用；1：启用）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBSls")
    public Result queryBSls(BSl param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bSlService.queryBSls(param));
    }

    @ApiOperation(value="删除课程", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bSlService.removeBSl(id);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="上传封面", notes="根据ID上传课程封面")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "file", value = "图片文件", required = false, dataType = "MultipartFile")})
    @PostMapping(value = "/uploadCover")
    public Result uploadCover(@RequestParam("file") MultipartFile cover,
                                    @RequestParam("id") String id) {
        String path = global.getBASE_UPLOAD_SERVER_LOCATION();
        String sl_logo_path = global.getIMAGE_LOCATION() + global.getLANCH_LOGO_LOCATION();
        Map<String, Object> uploadResult = UploadUtil.picImport(path, sl_logo_path, cover);
        if ((boolean) uploadResult.get(UploadUtil.IS_SUCCESSS)) {
            String url = (String) uploadResult.get(UploadUtil.URL);
            BSl bSl = new BSl();
            bSl.setSlCoverUrl(url);
            bSl.setBusinessId(id);
            PersistModel data = bSlService.updateBSlByPrimaryKey(bSl,getToken());
            return Result.builder(data,
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    id);
        } else
            throw new ServiceException(ResultServiceEnums.UPLOAD_FILE_ERROR);
    }

    @ApiOperation(value="修改课程简介", notes="修改课程简介")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "notes", value = "课程简介", required = false, dataType = "varchar")})
    @PostMapping(value = "/updateNotes")
    public Result updateNotes(@RequestParam("notes") String notes,
                                    @RequestParam("id") String id) {
        Result result = new Result();
        BSl bSl = new BSl();
        bSl.setSlNotes(notes);
        bSl.setBusinessId(id);
        PersistModel data = bSlService.updateBSlByPrimaryKey(bSl,getToken());
        result.setCode(ResultCode.SUCCESS.code);
        result.setSucessMessage(MessageConstant.MESSAGE_ALERT_SUCCESS,data);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="根据学号获取学习课程", notes="根据学号获取学习课程")
    @ApiImplicitParam(name = "lessonName", value = "课程名称关键字", required = false, dataType = "varchar")
    @GetMapping(value = "/mySlByStudentCode")
    public GridModel mySlByStudentCode(@RequestParam(value = "lessonName", defaultValue = "") String lessonName) {
        String code = "0108150216";//UserUtil.getUser().getLoginName();
        bSlService.queryStudentSLByCodeByPagination(getPaginationUtility(), code, lessonName);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="在教课程", notes="分页查询在教课程")
    @ApiImplicitParam(name = "lessonName", value = "课程名称关键字", required = false, dataType = "varchar")
    @GetMapping(value = "/mySlByTeacherCode")
    public GridModel mySlByTeacherCode(@RequestParam(value = "lessonName", defaultValue = "") String lessonName) {
        String code = "0108150216";//UserUtil.getUser().getLoginName();
        bSlService.queryTeacherSLByCodeByPagination(getPaginationUtility(), code, lessonName);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="课程详情", notes="根据ID查询课程详情")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/mySlDetailByIdWithoutAuth")
    public Result mySlDetailById(@RequestParam("id") String id) {
        SlDetailDTO data = bSlService.querySlDetailDTOById(id);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data);
    }

    @ApiOperation(value="精品课程", notes="分页查询精品课程")
    @GetMapping(value = "/queryBSlWithRecommendWithoutAuth")
    public GridModel queryBSlWithRecommend() {
        bSlService.queryBSlWithRecommendByPagination(getPaginationUtility());
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="查询学生是否存在课程中", notes="根据学生号及课程号查询该学生是否存在该课程中")
    @ApiImplicitParams({@ApiImplicitParam(name = "selectCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "studentCode", value = "学生学号", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryStuInLessionOrNot")
    public Result queryStuInLessionOrNot(@RequestParam("selectCode") String selectCode,
                                               @RequestParam("studentCode") String studentCode) {
        try {
            int data = bSlService.queryStuInLessionOrNot(selectCode,studentCode);
            if (data>0){
                return Result.builder(new PersistModel(1),"该学生选了该课程","该学生没有选该课程",1);
            }else{
                return Result.builder(new PersistModel(2),"该学生选了该课程","该学生没有选该课程",0);
            }
        }catch(Exception e){
            e.printStackTrace();
            return Result.builder(new PersistModel(2),"方法出错","方法出错","");
        }
    }


    @ApiOperation(value="判断是否是该老师的课", notes="根据教师Code和开课ID判断是否是该老师的课")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "teacherCode", value = "教师编号", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBslTeacher")
    public Result queryBslTeacher(@RequestParam("teacherCode") String teacherCode,
                                       @RequestParam("businessId") String businessId) {
        try {
            int data = bSlService.queryBslTeacher(teacherCode,businessId);
            if (data>0){
                return Result.builder(new PersistModel(1),"是该老师的课","不是该老师的课",true);
            }else{
                return Result.builder(new PersistModel(2),"是该老师的课","不是该老师的课",false);
            }
        }catch(Exception e){
            e.printStackTrace();
            return Result.builder(new PersistModel(2),"方法出错","方法出错","");
        }
    }


}
