package org.rcisoft.business.bsl.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bsl.dto.SlDTO;
import org.rcisoft.business.bsl.dto.SlDetailDTO;
import org.rcisoft.business.bsl.entity.BSl;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
* Created with CodeGenerator on 2017/07/12.
*/
@Repository
public interface BSlRepository extends BaseMapper<BSl> {

    /**
    * 根据条件查找
    * @param param
    * @return
    */
    @Select("<script>SELECT sl.business_id,sl.code,sl.lesson_code,lesson.lesson_name as lessonName,sl.teacher_code,u.name as teacherName,sl.is_online as isOnline,sl.sl_cover_url,sl.sl_notes,sl.is_recommend,sl.recommend_date,sl.recommend_order,sl.recommend_course FROM b_sl sl\n" +
            "LEFT JOIN b_lesson lesson ON sl.lesson_code=lesson.code \n" +
            "LEFT JOIN s_user u ON sl.teacher_code=u.login_name \n" +
            " <where>" +
            " sl.del_flag = '0' and sl.flag = '1'"+
            " <if test=\"isOnline !=NULL and isOnline !=''\">AND sl.is_online like CONCAT('%',#{isOnline},'%')</if>" +
            " <if test=\"teacherName !=null and teacherName !=''\">AND u.name like CONCAT('%',#{teacherName},'%')</if>" +
            " <if test=\"lessonName !=null and lessonName !=''\">AND lesson.lesson_name like CONCAT('%',#{lessonName},'%')</if>" +
            " <if test=\"termCode !=null and termCode !=''\">AND sl.term_code like CONCAT('%',#{termCode},'%')</if>" +
            " </where></script>")
    @ResultMap(value = "BaseResultMap" )
    List<BSl> queryBSls(Map<String, Object> param);

    /**
     * 获取精品课程
     * @return
     */
    @Select("<script>SELECT sl.business_id,sl.code,sl.lesson_code,lesson.lesson_name as lessonName,sl.teacher_code,u.name as teacherName,sl.is_online as isOnline,sl.sl_cover_url,sl.sl_notes,sl.is_recommend,sl.recommend_date,sl.recommend_order,sl.recommend_course, \n" +
            "(select COUNT(brs.student_code) from b_r_sl_student brs where brs.sl_code = sl.code) as studyNum FROM b_sl sl \n"+
            "LEFT JOIN b_lesson lesson ON sl.lesson_code=lesson.code \n" +
            "LEFT JOIN s_user u ON sl.teacher_code=u.login_name \n" +
            " <where>" +
            "sl.is_recommend = '1' and sl.is_online='1' and sl.del_flag = '0' and sl.flag = '1'"+
            " </where>" +
            " order by sl.recommend_order asc </script> ")
    @ResultMap(value = "BaseResultMap" )
    List<BSl> queryBSlsWithRecommend();

    /**
     * 批量插入
     * @param bSls
     * @return
     */
    @Insert("<script>INSERT INTO b_sl" +
            "(create_date,update_date,del_flag,flag,business_id,code,lesson_code,teacher_code,term_code,sl_cover_url,git_lab_project_id,git_lab_project_name)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "( #{item.createDate},#{item.updateDate},#{item.delFlag},#{item.flag},#{item.businessId},#{item.code},#{item.lessonCode},#{item.teacherCode},#{item.termCode},#{item.slCoverUrl},#{item.gitLabProjectId},#{item.gitLabProjectName})" +
            "</foreach></script>")
    int insertList(List<BSl> bSls);

    /**
     * 更新状态
     * @param lessonCode
     * @param online
     * @return
     */
    @Update("<script>update b_sl sl set sl.is_online=#{online} where sl.lessonCode=#{lessonCode}</script>")
    int updateBSls(String lessonCode, String online);

    /**
     * 根据学号获取学习课程
     * @param studentCode
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tt2.`code` as code,\n" +
            "\tt2.sl_cover_url as slCoverUrl,\n" +
            "\tt1.is_study as isStudy,\n" +
            "\tt2.business_id as businessId,\n" +
            "\tt3.lesson_name as lessonName\n" +
            "FROM\n" +
            "\tb_r_sl_student t1\n" +
            "LEFT JOIN b_sl t2 ON t1.sl_code = t2.`code`\n" +
            "LEFT JOIN b_lesson t3 ON t2.lesson_code = t3.`code`\n" +
            "WHERE\n" +
            "\tt1.student_code = #{studentCode}\n" +
            "AND t2.del_flag = 0\n" +
            "AND t2.flag = 1\n" +
            "AND t3.del_flag = 0\n" +
            "AND t3.flag = 1\n" +
            "AND t2.is_online = '1' "+
            "AND t3.lesson_name like #{lessonName}</script>")
    List<SlDTO> selectStudentSLByCode(@Param("studentCode") String studentCode, @Param("lessonName") String lessonName);

    /**
     * 根据工号获取在教课程
     * @param teacherCode
     * @param lessonName
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tt2.`code` as code,\n" +
            "\tt2.`is_online` as isOnline,\n" +
            "\tt2.sl_cover_url as slCoverUrl,\n" +
            "\tt2.business_id as businessId,\n" +
            "\tt3.lesson_name as lessonName\n" +
            "FROM\n" +
            "\tb_sl t2 \n" +
            "LEFT JOIN b_lesson t3 ON t2.lesson_code = t3.`code`\n" +
            "WHERE\n" +
            "\tt2.teacher_code = #{teacherCode}\n" +
            "AND t2.del_flag = 0\n" +
            "AND t2.flag = 1\n" +
            "AND t3.del_flag = 0\n" +
            "AND t3.flag = 1\n"+
            "AND t2.is_online != 2 "+
            "AND t3.lesson_name like #{lessonName}</script>")
    List<SlDTO> selectTeacherSLByCode(@Param("teacherCode") String teacherCode, @Param("lessonName") String lessonName);

    /**
     * 根据课程id获取开课详情
     * @param id
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tt1.business_id as businessId,\n" +
            "\tt1.`code` AS slCode,\n" +
            "\tt1.sl_notes as slNotes,\n" +
            "\tt1.sl_cover_url as coverUrl,\n" +
            "\tt6.lesson_name as lessonName,\n" +
            "\tt5.head_pic as teaPic,\n" +
            "\tt5.`name` AS teaName,\n" +
            "\tt5.t_num as teaSlNum,\n" +
            "\tt5.`code` AS teaCode,\n" +
            "\t(select count(*) from b_chapter bc where bc.sl_id=t1.business_id and bc.is_test like CONCAT('%','1','%' )) AS isTest,\n"+
            "\t(select count(*) from b_chapter bc where bc.sl_id=t1.business_id and bc.is_test like CONCAT('%','2','%' )) AS video\n"+
            "FROM\n" +
            "\tb_sl t1\n" +
            "LEFT JOIN b_lesson t6\n" +
            "ON t1.lesson_code = t6.`code`\n" +
            "LEFT JOIN (\n" +
            "\tSELECT\n" +
            "\t\tcount(t3.business_id) AS t_num,\n" +
            "\t\tt4.`name`,\n" +
            "\t\tt2.`code`,\n" +
            "\t\tt4.head_pic\n" +
            "\tFROM\n" +
            "\t\tb_teacher t2\n" +
            "\tLEFT JOIN b_sl t3 ON t2.`code` = t3.teacher_code\n" +
            "\tLEFT JOIN s_user t4 ON t4.login_name = t2.`code`\n" +
            "\tGROUP BY\n" +
            "\t\tt3.teacher_code\n" +
            ") t5 ON t5.`code` = t1.teacher_code\n" +
            "WHERE\n" +
            "\tt1.business_id = #{id}</script>")
    SlDetailDTO selectSlDetailDTOById(@Param("id") String id);

    /**
     * 根据教师id获取在教课程数
     * @param teaId
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tCOUNT(b1.`code`)\n" +
            "FROM\n" +
            "\tb_sl b1\n" +
            "LEFT JOIN b_teacher b2 ON b1.teacher_code = b2.`code`\n" +
            "WHERE\n" +
            "\tb2.business_id =#{teaId}</script>")
    int selectSlNumByStuent(String teaId);

    /**
     * 根据课程id获取开课数
     * @param lessonId
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tCOUNT(b1.`code`)\n" +
            "FROM\n" +
            "\tb_sl b1\n" +
            "LEFT JOIN b_lesson b2 ON b1.lesson_code = b2.`code`\n" +
            "WHERE\n" +
            "\tb2.business_id = #{lessonId}</script>")
    int selectslNumByLesson(String lessonId);

    /**
     * 根据课程id获取开课数
     * @param termId
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tCOUNT(b1.`code`)\n" +
            "FROM\n" +
            "\tb_sl b1\n" +
            "LEFT JOIN b_term b2 ON b1.term_code = b2.`code`\n" +
            "WHERE"+
            "\tb2.business_id = #{lessonId}</script>")
    int selectslNumByTermId(String termId);


    @Select("<script>SELECT `code` from b_sl where lesson_code = #{lessionCode} </script>")
    String queryCodeByLessionCode(String lessionCode);

    @Select("<script>SELECT count(*) from b_r_sl_student where sl_code = #{code} and student_code = #{studentCode} </script>")
    int queryStuInLessionOrNot(String studentCode, String code);

    @Select("<script>SELECT count(*) from b_sl where teacher_code = #{0} and business_id = #{1} </script>")
    int queryBslTeacher(String teacherCode, String businessId);
}

