package org.rcisoft.business.bsl.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.brslstudent.dao.BRSlStudentRepository;
import org.rcisoft.business.brslstudent.entity.BRSlStudent;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.bsl.dto.SlDTO;
import org.rcisoft.business.bsl.dto.SlDetailDTO;
import org.rcisoft.business.bsl.entity.BSl;
import org.rcisoft.business.bsl.enums.BSlEnum;
import org.rcisoft.business.bsl.enums.RecommendEnum;
import org.rcisoft.business.bsl.service.BSlService;
import org.rcisoft.business.bteacher.dao.BTeacherRepository;
import org.rcisoft.business.bteacher.entity.BTeacher;
import org.rcisoft.business.bterm.dao.BTermRepository;
import org.rcisoft.business.bterm.entity.BTerm;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.component.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BSlServiceImpl implements BSlService {
    @Autowired
    private BSlRepository bSlRepository;
    @Autowired
    private BTeacherRepository bTeacherRepository;
    @Autowired
    private BLessonRepository bLessonRepository;
    @Autowired
    private BTermRepository bTermRepository;
    @Autowired
    private BRSlStudentRepository brSlStudentRepository;
    /*@Autowired
    private UserRepository userRepository;
    @Autowired
    private IAction action;*/
    @Autowired
    private Global global;

    @Override
    public BSl selectOne(String businessId){
        return bSlRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BSl> queryBSlsByPagination(PageUtil pageUtil, BSl model) {
        return bSlRepository.queryBSls(getParames(model));
    }

    @Override
    public List<BSl> queryBSls(BSl model) {

        return bSlRepository.queryBSls(getParames(model));
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel persist(BSl model,String token) throws UnsupportedEncodingException {
        int line = 0;
        if(model.getBusinessId()!=null){
            BSl bSl = bSlRepository.selectByPrimaryKey(model.getBusinessId());
            if(bSl==null)
                throw new ServiceException(ResultServiceEnums.SL_INFO_NOT_EXISTS);
            boolean isRecommend = model.getIsRecommend()!=null&&model.getIsRecommend().equals(RecommendEnum.RECOMMEND.getCode());
            boolean isOnline = bSl.getIsOnline().equals(BSlEnum.START.getCode());
            if(isRecommend&&!isOnline)
                throw new ServiceException(ResultServiceEnums.SL_NOT_ONLINE);

            UserUtil.setCurrentMergeOperation(model,token);
            line = bSlRepository.updateByPrimaryKeySelective(model);
        }
        else {
            String lecode = model.getLessonCode();
            BLesson bLesson = bLessonRepository.queryBLessonByCode(lecode);
            if(bLesson==null)
                throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
            model.setSlCoverUrl(bLesson.getDefaultUrl());

            if(bSlRepository.selectOne(new BSl(model.getCode()))!=null)
                throw new ServiceException(ResultServiceEnums.SL_EXISTS);
            //3. 封装git lab
            /*User user = userRepository.selectOne(new User(model.getTeacherCode()));
            AccountDTO dto =  new AccountDTO(user.getGitLabId(),user.getGitLabImpressionToken());
            dto.setUsername(user.getGitLabUsername());
            if(StringUtils.isEmpty(dto.getId())|| StringUtils.isEmpty(dto.getImpersonationToken())){
                throw new ServiceException(ResultServiceEnums.TEACHER_NOT_HAVA_GIT_LAB);
            }
            Project project = action.createProject(pName,dto);
            model.setGitLabProjectId(project.getId());*/
            String pName = global.getGIT_LAB_PROJECT_PREFIX()+model.getCode();
            model.setGitLabProjectName(pName);
            UserUtil.setCurrentPersistOperation(model,token);
            line = bSlRepository.insertSelective(model);
        }
        //FamilyLogger.sysInfo(UserUtils.getUser().getBusinessId(),UserUtils.getUser().getLoginName()+"新增了ID为"+user.getBusinessId()+"的用户");
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel importExcel(HSSFWorkbook hwb,String token) throws IOException {
        ArrayList<BSl> bSls = new ArrayList<BSl>();
        ArrayList<String> repeatCode = new ArrayList<String>();
        ArrayList<String> lessonCode = new ArrayList<String>();
        ArrayList<String> teacherCode = new ArrayList<String>();
        ArrayList<String> gitlab = new ArrayList<String>();
        ArrayList<String> termCode = new ArrayList<String>();
        List<BSl> repeatSl = new ArrayList<>();

        String[] headers = {"课序号","课程编号","教师工号","学期编号"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb,headers);  //获取excel数据

        if (values.size()<0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        for(String[] value:values){   //将数据封装到entity
            if(value.length>=4) {
                //1. 检查数据正确性
                BSl bSl = null;
                BSl b = new BSl(value[0]);
                BSl bSl1 = new BSl();
                bSl1.setTermCode(value[3]);
                bSl1.setTeacherCode(value[2]);
                bSl1.setLessonCode(value[1]);
                if (value[0] == null || value[0].equals("") || value[1] == null || value[1].equals("")) {
                    continue;
                }
                if ((bSl = bSlRepository.selectOne(b)) != null) {
                    repeatCode.add(value[0]);
                    continue;
                }
                //
                if ((bTeacherRepository.selectOne(new BTeacher(value[2]))) == null) {
                    if (!teacherCode.contains(value[2]))
                        teacherCode.add(value[2]);
                    continue;
                }
                if ((bLessonRepository.queryBLessonByCode(value[1])) == null) {
                    if (!lessonCode.contains(value[1]))
                        lessonCode.add(value[1]);
                    continue;
                }
                if ((bTermRepository.selectOne(new BTerm(value[3]))) == null) {
                    if (!termCode.contains(value[3]))
                        termCode.add(value[3]);
                    continue;
                }
                if ( bSlRepository.selectOne(bSl1) != null) {
                    repeatSl.add(bSl1);
                    continue;
                }

                //2 封装BSL
                bSl = new BSl(value[0], value[1], value[2], value[3]);
                bSl.setSlCoverUrl(bLessonRepository.queryBLessonByCode(value[1]).getDefaultUrl());
                UserUtil.setCurrentPersistOperation(bSl,token);

                //3. 封装git lab
                /*User user = userRepository.selectOne(new User(value[2]));
                AccountDTO dto =  new AccountDTO(user.getGitLabId(),user.getGitLabImpressionToken());
                dto.setUsername(user.getGitLabUsername());
                if(StringUtils.isEmpty(dto.getId())|| StringUtils.isEmpty(dto.getImpersonationToken())){
                    gitlab.add(value[2]);
                    continue;
                }
                Project project = action.createProject(pName,dto);
                bSl.setGitLabProjectId(project.getId());*/
                String pName = global.getGIT_LAB_PROJECT_PREFIX()+value[0];
                bSl.setGitLabProjectName(pName);
                if (!bSls.contains(bSl)) {
                    bSls.add(bSl);
                }
            }
        }
        String result ="";
        if(bSls.size()>0){
            int line = bSlRepository.insertList(bSls);
            if (line>0){
                result  = "成功导入"+bSls.size()+"条数据。";
            }else{
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR);
            }
        }

        StringBuilder nullResult = new StringBuilder();
        if(repeatSl.size()>0){
            nullResult.append("以下开课记录已存在：").append(JSON.toJSONString(repeatSl)).append("。 ");
        }
        if(repeatCode.size()>0){
            nullResult.append("以下课序号已存在：").append(JSON.toJSONString(repeatCode)).append("。 ");
        }
        if(lessonCode.size()>0){
            nullResult.append("以下课程编号不存在：").append(JSON.toJSONString(lessonCode)).append("。 ");
        }
        if(teacherCode.size()>0){
            nullResult.append("以下教师工号不存在：").append(JSON.toJSONString(teacherCode)).append("。 ");
        }
        if(termCode.size()>0){
            nullResult.append("以下学期编号不存在：").append(JSON.toJSONString(termCode)).append("。 ");
        }
        if(gitlab.size()>0){
            nullResult.append("以下教师的gitlab账户不存在（请重试新增该教师）：").append(JSON.toJSONString(gitlab));
        }
        result+=nullResult;
        if(bSls.size()<1){

            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR.getCode(),result);
        }




        return new PersistModel(1,result);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel removeBSl(String id) {
        BSl bSl = bSlRepository.selectByPrimaryKey(id);
        if(bSl==null)
            throw new ServiceException(ResultServiceEnums.SL_INFO_NOT_EXISTS);
        //action.deleteProject(bSl.getGitLabProjectId());
        brSlStudentRepository.deleteBatch(bSl.getCode());
        int line = bSlRepository.deleteByPrimaryKey(id);
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel updateIsOnline(String lessonCode,String token) {
        BSl bSl = new BSl();
        bSl.setCode(lessonCode);
        BSl one = bSlRepository.selectOne(bSl);
        if(one==null)
            throw new ServiceException(ResultServiceEnums.SL_INFO_NOT_EXISTS);
        if(one.getIsOnline().equals(BSlEnum.START.getCode()))
            one.setIsOnline(BSlEnum.UN_START.getCode());
        else
            one.setIsOnline(BSlEnum.START.getCode());
        UserUtil.setCurrentMergeOperation(one,token);
        int result = bSlRepository.updateByPrimaryKeySelective(one);
        return new PersistModel(result);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel updateBSlByPrimaryKey(BSl bSl, String token) {
        UserUtil.setCurrentMergeOperation(bSl,token);
        int line = bSlRepository.updateByPrimaryKeySelective(bSl);
        return new PersistModel(line);
    }

    @Override
    public List<SlDTO> queryStudentSLByCodeByPagination(PageUtil pageUtil, String code, String lessonName) {
        return bSlRepository.selectStudentSLByCode(code,paramHandler(lessonName));
    }

    @Override
    public List<SlDTO> queryTeacherSLByCodeByPagination(PageUtil pageUtil, String code, String lessonName) {
        return bSlRepository.selectTeacherSLByCode(code,paramHandler(lessonName));
    }

    @Override
    public List<BSl> queryBSlWithRecommendByPagination(PageUtil pageUtil) {
        return bSlRepository.queryBSlsWithRecommend();
    }

    @Override
    public SlDetailDTO querySlDetailDTOById(String id) {
        return bSlRepository.selectSlDetailDTOById(id);
    }

    @Override
    public boolean stuHasSl(String stuCode, String slCode) {
        BRSlStudent brSlStudent = new BRSlStudent();
        brSlStudent.setSlCode(slCode);
        brSlStudent.setStudentCode(stuCode);
        return brSlStudentRepository.selectOne(brSlStudent)!=null;
    }

    @Override
    public boolean teaHasSl(String teaCode, String slCode) {
        BSl bSl = new BSl();
        bSl.setCode(slCode);bSl.setTeacherCode(teaCode);
        return bSlRepository.selectOne(bSl)!=null;
    }

    @Override
    public int queryStuInLessionOrNot(String selectCode, String studentCode) {
        int result = bSlRepository.queryStuInLessionOrNot(studentCode,selectCode);
        return result;
    }

    @Override
    public int queryBslTeacher(String teacherCode, String businessId) {
        return bSlRepository.queryBslTeacher(teacherCode,businessId);
    }

    private Map<String,Object> getParames(BSl model){
        Map param = new HashMap<String,Object>();
        param.put("lessonName", model.getLessonName());
        param.put("teacherName",model.getTeacherName());
        param.put("isOnline", model.getIsOnline());
        param.put("termCode", model.getTermCode());
        return param;
    }

    private String paramHandler(String param){
        return "%"+param+"%";
    }

}
