package org.rcisoft.business.bvideo.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.IOUtils;
import org.rcisoft.business.bvideo.entity.BVideo;
import org.rcisoft.business.bvideo.service.BVideoService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.List;

/**
* Created by gaowenfneg on 2017-10-9 14:34:56.
*/
@RestController
@RequestMapping("/BVideo")
public class BVideoController extends PaginationController<BVideo> {
    @Autowired
    private BVideoService bVideoService;
    @Autowired
    private Global global;


    @ApiOperation(value="添加/编辑视频", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "videoName", value = "视频名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "videoUrl", value = "视频地址", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "'0'为视频，'1'为附件", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(BVideo bVideo) {
        PersistModel data = bVideoService.persist(bVideo,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bVideo);
    }

    @ApiOperation(value="逻辑删除", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(String id) {
        PersistModel data = bVideoService.removeBVideo(id,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(String businessId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bVideoService.selectOne(businessId));
    }


    @ApiOperation(value="根据条件分页查询", notes="根据条件分页查询视频信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "videoName", value = "视频名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "videoUrl", value = "视频地址", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "'0'为视频，'1'为附件", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBVideosByPagination")
    public GridModel queryBVideosByPagination(BVideo param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bVideoService.queryBVideosByPagination(getPaginationUtility(),param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="根据条件分页查询", notes="根据条件分页查询视频信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "videoName", value = "视频名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "videoUrl", value = "视频地址", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "'0'为视频，'1'为附件", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBVideos")
    public Result queryBVideos(BVideo param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bVideoService.queryBVideos(param));
    }


    @ApiOperation(value="上传视频", notes="上传视频")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "videoName", value = "视频名称", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "videoUrl", value = "视频地址", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "type", value = "'0'为视频，'1'为附件", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "createBy", value = "创建者", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "createDate", value = "创建日期", required = false, dataType = "datetime"),
            @ApiImplicitParam(name = "updateBy", value = "更新者", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "updateDate", value = "更新日期", required = false, dataType = "datetime"),
            @ApiImplicitParam(name = "delFlag", value = "删除标记（0：正常；1：删除；2：审核）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "flag", value = "启用标记（0：停用；1：启用）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "remarks", value = "备注", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "file", value = "视频文件", required = false, dataType = "MultipartFile")})
    @PostMapping(value = "/uploadVideo")
    public Result uploadVideo(BVideo bVideo, HttpServletRequest request) {
        Result result = new Result();
        List<MultipartFile> list = ((MultipartHttpServletRequest)request).getFiles("file");
        try{
            String pathName;
            if (bVideo.getType().equals("0")){
                pathName = global.getVIDEO_LOCATION();//视频路径
            }else{
                pathName = global.getFILE_LOCATION();//文件路径
            }

            String path= global.getBASE_UPLOAD_SERVER_LOCATION()
                        + pathName
                        + File.separator + bVideo.getLessonId() + File.separator;//根据开课ID区分文件
            File outFile=new File(path);

            if(!outFile.exists()){//判断保存路径是否存在，不存在新建
                outFile.mkdirs();
            }
            String videoUrl = bVideoService.uploadVideoToServer(list,bVideo,path,getToken());
            return Result.builder(new PersistModel(1),
                    "上传成功",
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    videoUrl);


        }catch (Exception e){
            e.printStackTrace();
            return Result.builder(new PersistModel(2),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    "");

        }
    }

    /**
     * 下载文件
     * @param chapterId
     * @param response
     * @return
     */
    @ApiOperation(value="下载文件", notes="下载文件")
    @ApiImplicitParam(name = "chapterId", value = "章节ID", required = false, dataType = "varchar")
    @PostMapping(value = "/downloadFile")
    public Result downloadFile(String chapterId, HttpServletResponse response){
        BVideo video = bVideoService.queryFileUrlByChapterId(chapterId);
        BufferedInputStream bis = null;
        BufferedOutputStream bos = null;
        if(video.getVideoUrl()!=""&&video.getVideoUrl()!=null){
            try {
                File file=new File(global.getBASE_UPLOAD_SERVER_LOCATION()+video.getVideoUrl());
                bis = new BufferedInputStream(new FileInputStream(file));
                bos = new BufferedOutputStream(response.getOutputStream());

                response.reset();
                response.setContentType("application/octet-stream;charset=utf-8");
                response.setHeader("Content-Disposition", "attachment;filename="+ new String((video.getVideoName()).getBytes(), "iso-8859-1"));
                response.setHeader("Access-Control-Allow-Origin", "*");
                IOUtils.copy(bis, bos);
            }catch (Exception e){
                e.printStackTrace();
            }finally {
                if (bis != null)
                    try {
                        bis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                if (bos != null)
                    try {
                        bos.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
            }
        }
        return Result.builder(new PersistModel(2),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                "");
    }
}

