package org.rcisoft.business.bvideo.service.impl;

import org.rcisoft.business.bvideo.dao.BVideoRepository;
import org.rcisoft.business.bvideo.entity.BVideo;
import org.rcisoft.business.bvideo.service.BVideoService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.component.PageUtil;
import org.rcisoft.core.constant.DelStatus;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by gaowenfneg on 2017-10-9 14:34:56.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BVideoServiceImpl implements BVideoService {
    @Autowired
    private BVideoRepository bVideoRepository;
    @Autowired
    private Global global;

    @Override
    public BVideo selectOne(String businessId){
        return bVideoRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BVideo> queryBVideosByPagination(PageUtil pageUtil, BVideo model) {
        Map param = new HashMap<String,Object>();
        return bVideoRepository.queryBVideos(param);
    }

    @Override
    public List<BVideo> queryBVideos(BVideo model) {
        Map param = new HashMap<String,Object>();
        return bVideoRepository.queryBVideos(param);
    }


    @Override
    public PersistModel persist(BVideo model, String token){
        UserUtil.setCurrentPersistOperation(model,token);
        int line = bVideoRepository.insertSelective(model);
        bVideoRepository.insertChapterVideo(model);
        return new PersistModel(line);
    }

    @Override
    public PersistModel removeBVideo(String id, String token) {
        BVideo bVideo = bVideoRepository.selectByPrimaryKey(id);
        bVideo.setDelFlag(DelStatus.DELETED.getStatus());
        UserUtil.setCurrentMergeOperation(bVideo,token);
        int line = bVideoRepository.updateByPrimaryKeySelective(bVideo);
        return new PersistModel(line);
    }

    @Override
    public String uploadVideoToServer(List<MultipartFile> list, BVideo bVideo, String path, String token) {

        String videoUrl = "";

        if (list.size()>0) {
            for (int i = 0; i < list.size(); ++i) {
                MultipartFile file = list.get(i);
                try {
                    //查询此节是否已有视频或文件
                    BVideo video = bVideoRepository.selectInfoByChapterId(bVideo.getChapterId(),bVideo.getType());

                    if (video!=null){//此节已有视频或文件，将已有的视频或文件放置到temp目录下，然后上传新视频或文件
                        String oldPath = global.getBASE_UPLOAD_SERVER_LOCATION() + video.getVideoUrl();

                        this.removeVideoToTemp(oldPath,video.getVideoName(),video.getType());//复制之前视频到temp/video文件下

                        BVideo newVideo = this.uploadVideo(file,bVideo,path);//上传新视频
                        newVideo.setBusinessId(video.getBusinessId());
                        UserUtil.setCurrentMergeOperation(newVideo,token);
                        bVideoRepository.updateByPrimaryKeySelective(newVideo);//更新表中信息
                        videoUrl = newVideo.getVideoUrl();
                    }else{
                        BVideo newVideo = this.uploadVideo(file,bVideo,path);//上传新视频
                        this.persist(newVideo,token);//插入表信息
                        videoUrl = newVideo.getVideoUrl();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new RuntimeException(e);
                }
            }
        }
        return videoUrl;
    }




    /**
     * 将原视频复制到temp/video文件下，并删除原视频
     * @param oldPath
     * @param videoName
     */
    private void removeVideoToTemp(String oldPath, String videoName, String type) {
        String pathName;
        if (type=="0"){
            pathName = global.getVIDEO_LOCATION();//视频路径
        }else{
            pathName = global.getFILE_LOCATION();//文件路径
        }

        String tempPath = global.getBASE_UPLOAD_SERVER_LOCATION()
                + global.getTEMP_LOCATION()
                + pathName;

        File filePath = new File(tempPath);
        if(!filePath.exists()){
            filePath.mkdirs();
        }
        try {
            int bytesum = 0;
            int byteread = 0;
            File oldfile = new File(oldPath);
            if (oldfile.exists()) { //文件存在时
                InputStream inStream = new FileInputStream(oldPath); //读入原文件
                FileOutputStream fs = new FileOutputStream(tempPath+ File.separator+videoName);
                byte[] buffer = new byte[1024000];
                int length;
                while ( (byteread = inStream.read(buffer)) != -1) {
                    bytesum += byteread; //字节数 文件大小
                    fs.write(buffer, 0, byteread);
                }
                inStream.close();
                fs.close();
                oldfile.delete();
            }
        }
        catch (Exception e) {
            System.out.println("出错");
            e.printStackTrace();

        }
    }

    /**
     * 上传视频到video目录
     * @param file
     * @param bVideo
     * @param path
     * @return
     * @throws IOException
     */
    private BVideo uploadVideo(MultipartFile file, BVideo bVideo, String path) throws IOException {

        BufferedOutputStream stream = null;

        String pathName;
        if (bVideo.getType().equals("0")){
            pathName = global.getVIDEO_LOCATION();//视频路径
        }else{
            pathName = global.getFILE_LOCATION();//文件路径
        }

        //上传文件
        String uploadFile = file.getOriginalFilename();

        //文件的真实名称
        String name = uploadFile.substring(uploadFile.lastIndexOf(File.separator) + 1,uploadFile.lastIndexOf('.'));

        // 截取上传文件的后缀
        String uploadFileSuffix = uploadFile.substring(uploadFile.lastIndexOf('.') + 1, uploadFile.length());

        // 设置文件名为乱码
        String uploadFileName = IdGen.uuid();

        String filename = uploadFileName + "." + uploadFileSuffix;

        stream = new BufferedOutputStream(new FileOutputStream(new File(path + filename)));
        byte[] bytes = file.getBytes();
        stream.write(bytes,0,bytes.length);

        String newPath = pathName
                + File.separator + bVideo.getLessonId() + File.separator;

        bVideo.setVideoName(name+ "." +uploadFileSuffix);//文件名
        bVideo.setVideoUrl(newPath+ filename);//存储地址
        stream.close();
        return bVideo;
    }

    @Override
    public BVideo queryFileUrlByChapterId(String chapterId) {
        return bVideoRepository.queryFileUrlByChapterId(chapterId);
    }
}
